/*
 *   ScenarLang a domain specific language that attempts to make it even easier
 *   to write unit tests with mockito
 *    
 *   Copyright (C) 2012  X. Gillard 
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *   
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package xgi.ut.dsl.utl.filtering;

import static xgi.ut.dsl.utl.ReflectionUtil.*;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;

public class SignatureMatchFilter implements Filter<Method> {
	private final String name;
	private final List<Class<?>> argsType;
	private final Class<?> resultType;
	
	public SignatureMatchFilter(final String name, final List<Class<?>> argsType, final Class<?> resultType) {
		this.name = name;
		this.argsType = argsType;
		this.resultType = resultType;
	}

	@Override
	public boolean isCompliant(Method m) {
		return signatureMatch(m, name, argsType, resultType);
	}
	
	@Override
	public String toString(){
		argsString();
		return String.format("%1$s %2$s(%3$s)", resultType.getCanonicalName(), name, argsString());
	}
	
	/** returns true if the signature matches */
	private boolean signatureMatch(Method m, String name, List<Class<?>> argsType, Class<?> resultType) {
		return m.getName().equals(name) &&
		argumentsTypeMatch(m.getParameterTypes(), argsType) &&
		(resultType==null || isAssignableFrom(m.getReturnType(), resultType));
	}
	
	private String argsString() {
		StringBuffer args = new StringBuffer();
		Iterator<Class<?>> it = argsType.iterator();
		while(it.hasNext()){
			args.append(it.next().getCanonicalName());
			if(it.hasNext()){
				args.append(", ");
			}
		}
		return args.toString();
	}
}