/*
 *   ScenarLang a domain specific language that attempts to make it even easier
 *   to write unit tests with mockito
 *    
 *   Copyright (C) 2012  X. Gillard 
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *   
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package xgi.ut.dsl.utl.overloading;

import java.lang.reflect.Method;
import java.util.Comparator;

import xgi.ut.dsl.utl.ReflectionUtil;

public class MethodSpecificityComparator implements Comparator<Method> {
	@Override
	public int compare(Method m1, Method m2) {
		if (isMoreSpecific(m1, m2)) {
			return -1;
		}
		if (isMoreSpecific(m2, m1)) {
			return 1;
		}
		return 0;
	}

	public static boolean isMoreSpecific(Method m1, Method m2) {
		Class<?>[] args1 = m1.getParameterTypes();
		Class<?>[] args2 = m2.getParameterTypes();

		int count = 0;
		for (int i = 0; i < args1.length; i++) {
			if (ReflectionUtil.isAssignableFrom(args2[i], args1[i])) {
				count++;
			}
		}

		return count == args1.length;
	}
}